MODULE zdfsh2
   !!======================================================================
   !!                       ***  MODULE  zdfsh2  ***
   !! Ocean physics:  shear production term of TKE 
   !!=====================================================================
   !! History :   -   !  2014-10  (A. Barthelemy, G. Madec)  original code
   !!   NEMO     4.0  !  2017-04  (G. Madec)  remove u-,v-pts avm
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   zdf_sh2       : compute mixing the shear production term of TKE
   !!----------------------------------------------------------------------
   USE oce            ! ocean: shared variables
   USE dom_oce        ! domain: ocean
   USE zdf_oce        ! vertical physics: variables
   !
   USE in_out_manager ! I/O manager
   USE lib_mpp        ! MPP library
   USE timing         ! Timing

   IMPLICIT NONE
   PRIVATE

   PUBLIC   zdf_sh2        ! called by zdftke, zdfglf, and zdfric
   
   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2017)
   !! $Id: $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE zdf_sh2(  psh2  ) 
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE zdf_sh2  ***
      !!
      !! ** Purpose :   Compute the shear production term of a TKE equation
      !!
      !! ** Method  : - a stable discretization of this term is linked to the
      !!                time-space discretization of the vertical diffusion
      !!                of the OGCM. NEMO uses C-grid, a leap-frog environment 
      !!                and an implicit computation of vertical mixing term,
      !!                so the shear production at w-point is given by:
      !!                   sh2 = mi[   mi(avm) * dk[ub]/e3ub * dk[un]/e3un   ] 
      !!                       + mj[   mj(avm) * dk[vb]/e3vb * dk[vn]/e3vn   ] 
      !!                NB: wet-point only horizontal averaging of shear
      !!
      !! ** Action  : - psh2 shear prod. term at w-point (interior ocean domain only)
      !!
      !! References :   Bruchard, OM 2002
      !! ---------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(  out) ::   psh2   ! shear production of TKE (w-points)
      !
      INTEGER  ::   ji, jj, jk   ! dummy loop arguments
      REAL(wp), DIMENSION(jpi,jpj) ::   zsh2u, zsh2v   ! 2D workspace
      !!--------------------------------------------------------------------
      IF( nn_timing == 1 )  CALL timing_start('zdf_sh2')
      !
      DO jk = 2, jpkm1
         DO jj = 1, jpjm1        !* Shear production at uw- and vw-points (energy conserving form)
            DO ji = 1, jpim1
               zsh2u(ji,jj) = 0.5 * ( avm(ji+1,jj,jk) + avm(ji,jj,jk) ) * wumask(ji,jj,jk)  &
                  &               * (  un(ji,jj,jk-1) -  un(ji,jj,jk) )   &
                  &               * (  ub(ji,jj,jk-1) -  ub(ji,jj,jk) ) / ( e3uw_n(ji,jj,jk) * e3uw_b(ji,jj,jk) )
               zsh2v(ji,jj) = 0.5 * ( avm(ji,jj+1,jk) + avm(ji,jj,jk) ) * wvmask(ji,jj,jk)  &
                  &               * (  vn(ji,jj,jk-1) -  vn(ji,jj,jk) )   &
                  &               * (  vb(ji,jj,jk-1) -  vb(ji,jj,jk) ) / ( e3vw_n(ji,jj,jk) * e3vw_b(ji,jj,jk) )
            END DO
         END DO
         DO jj = 2, jpjm1        !* shear production at w-point
            DO ji = 2, jpim1
               !
!!gm original type of coding
!              psh2(ji,jj,jk) = ( zsh2u(ji-1,jj) + zsh2u(ji,jj) ) / MAX( 1._wp , umask(ji-1,jj,jk)+umask(ji,jj,jk) )   &
!                 &           + ( zsh2v(ji,jj-1) + zsh2v(ji,jj) ) / MAX( 1._wp , vmask(ji,jj-1,jk)+vmask(ji,jj,jk) )    
!
!!gm optimized but it changes the last digits after 1 year of GYRE
               !                       ! coast mask: =1 at the coast ; =2 otherwise
               !                       ! no need of wmask as zsh2 are already masked
               psh2(ji,jj,jk) = ( zsh2u(ji-1,jj) + zsh2u(ji,jj) ) * ( 2._wp - umask(ji-1,jj,jk) * umask(ji,jj,jk) )   &
                  &           + ( zsh2v(ji,jj-1) + zsh2v(ji,jj) ) * ( 2._wp - vmask(ji,jj-1,jk) * vmask(ji,jj,jk) )
            END DO
         END DO
      END DO 
      !
      IF( nn_timing == 1 )  CALL timing_stop('zdf_sh2')     
      !
   END SUBROUTINE zdf_sh2

   !!======================================================================
END MODULE zdfsh2
